using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using LibUSB_Interface;


namespace MoodlightNS
{
    public partial class Moodlight : Form
    {
        public const int MAX_COL_AM = 18; //80 / 18
        public const int PACKET_LENGTH = 61; //247 / 61
        public libusb_intf usb = new libusb_intf();
         
        public Moodlight()
        {
            libusb_intf.usb_set_debug(40);
            InitializeComponent();
            Application.ApplicationExit += new EventHandler(exit);
            
        }

        Timer t = new Timer();

        private void exit(object sender, EventArgs e) {
            updateLight(true);
            if (usb.device != IntPtr.Zero) {
                libusb_intf.usb_close(usb.device);
            }
        }


        private void kt_Scroll(object sender, EventArgs e)
        {
            kl.Text = kt.Value.ToString();
            updateLight(false);
        }

        private void nt_Scroll(object sender, EventArgs e)
        {
            nl.Text = nt.Value.ToString();
            updateLight(false);
        }



        private void button1_Click(object sender, EventArgs e)
        {
            if (listBox1.Items.Count >= MAX_COL_AM)
            {
                MessageBox.Show("You can have a maximum of "+MAX_COL_AM+" colors.");
                return;
            }
            ColorDialog cd = new ColorDialog();
            DialogResult dr = cd.ShowDialog();
            if (dr == DialogResult.OK)
            {
                Color c = cd.Color;
               
                listBox1.Items.Add(c.R + ", " + c.G + ", " + c.B);
            }
            pictureBox1.Invalidate();
            updateLight(false);
        }

        private void redraw(Graphics gr)
        {


            for (int i = 0; i < listBox1.Items.Count; i++)
            {
                int indexOfItem = listBox1.IndexFromPoint(0,i*listBox1.ItemHeight);
                if (indexOfItem >= 0 && indexOfItem < listBox1.Items.Count)
                {
                    String item = listBox1.Items[indexOfItem].ToString();
                    char[] splitter = { ',' };
                    String[] pcs = item.Split(splitter);

                    int r = int.Parse(pcs[0]);
                    int g = int.Parse(pcs[1]);
                    int b = int.Parse(pcs[2]);

                    gr.FillRectangle(new SolidBrush(Color.FromArgb(r, g, b)), 3,
                        i * listBox1.ItemHeight + 3 ,
                        pictureBox1.Width - 10, listBox1.ItemHeight - 6);
                }
            }
        }

        private void updateLightToEEPROM(object sender, EventArgs eArgs)
        {
            updateLight(true);
        }

        private void updateLight(bool toEEPROM)
        {
            if (!toEEPROM)
            {
                t.Stop();
                t = new Timer();
                t.Interval = 2000;
                t.Start();
                t.Tick += new EventHandler(updateLightToEEPROM);
            }
            byte[] data = new byte[PACKET_LENGTH];
            data[0] = 6;
            if (toEEPROM == true)
            {
                t.Stop();
                data[0] = 8;
            }
            data[4] = (byte)(255-nt.Value);
            data[5] = (byte)kt.Value;
            data[6] = (byte)listBox1.Items.Count;
            for (int i = 0; i < listBox1.Items.Count; i++)
            {
                String item = listBox1.Items[i].ToString();
                char[] splitter = { ',' };
                String[] pcs = item.Split(splitter);
                for (int k = 0; k < 3; k++)
                {
                    data[i * 3 + k+7] = byte.Parse(pcs[k]);
                }
            }
            usb.usbSend(data,4);
        }


        // USB CODE:


        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            redraw(e.Graphics);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            string str = usb.initUSB(5824, 1007);
            if (str != null)
            {
                MessageBox.Show("No moodlight device found...");
                Application.Exit();
            }
            else
            {
                byte[] data = new byte[PACKET_LENGTH];
                data[0] = 7;
                byte[] reply = usb.usbSend(data, PACKET_LENGTH);
                nt.Value = 255 - reply[4];
                kt.Value = reply[5];
                byte colorCount = reply[6];
                if (reply.Length != PACKET_LENGTH || colorCount > MAX_COL_AM)
                {
                    MessageBox.Show("Replylen or CC wrong. not PACKET_LENGTH:" + reply.Length + " or not <"+MAX_COL_AM+":" + colorCount + " data:" + usb.arrayToString(reply));
                    return;
                }
                for (int i = 0; i < colorCount; i++)
                {
                    listBox1.Items.Add(reply[i * 3+7] + ", " + reply[i * 3 + 8] + ", " + reply[i * 3 + 9]);
                }
                pictureBox1.Invalidate();
                kl.Text = kt.Value.ToString();
                nl.Text = nt.Value.ToString();
            }

        }

        private void button2_Click(object sender, EventArgs e)
        {
            for (int i = listBox1.SelectedIndices.Count-1; i >= 0;i-- )
            {
                listBox1.Items.RemoveAt(listBox1.SelectedIndices[i]);
            }
            pictureBox1.Invalidate();
            updateLight(false);

        }

        private int startDragItem;

        private void listBox1_MouseDown(object sender, MouseEventArgs e)
        {
            int indexOfItem = listBox1.IndexFromPoint(e.X, e.Y);
            if (indexOfItem >= 0 && indexOfItem
              < listBox1.Items.Count)  // check we clicked down on a string
            {
                startDragItem = indexOfItem;
                listBox1.DoDragDrop(listBox1.Items[indexOfItem], DragDropEffects.Move);
            }
        }

        private void listBox1_DragDrop(object sender, DragEventArgs e)
        {
        }

        private void listBox1_DragEnter(object sender, DragEventArgs e)
        {
            //MessageBox.Show("drag enter");
            e.Effect = DragDropEffects.Move;
        }



        private void listBox1_DragOver(object sender, DragEventArgs e)
        {
            listBox1.Items.RemoveAt(startDragItem);
            if (e.Data.GetDataPresent(DataFormats.StringFormat))
            {
                int indexOfItemUnderMouseToDrop =
                    listBox1.IndexFromPoint(listBox1.PointToClient(
                    new Point(e.X, e.Y)));
                if (indexOfItemUnderMouseToDrop >= 0 &&
                   indexOfItemUnderMouseToDrop < listBox1.Items.Count)
                {
                    listBox1.Items.Insert(indexOfItemUnderMouseToDrop,
                     e.Data.GetData(DataFormats.Text));
                    startDragItem = indexOfItemUnderMouseToDrop;
                }
                else
                {
                    // add the selected string to bottom of list
                    listBox1.Items.Add(e.Data.GetData(DataFormats.Text));
                    startDragItem = listBox1.Items.Count - 1;
                }

            }
            pictureBox1.Invalidate();
            updateLight(false);

        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            pictureBox1.Invalidate();
        }

        private void listBox1_Scroll(object Sender, BetterListBox.BetterListBoxScrollArgs e)
        {
            pictureBox1.Invalidate();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            new Form2(this).ShowDialog(this);
        }

  
      

    }
}