using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using LibUSB_Interface;

namespace Oscilloscope
{
    public partial class Scope : Form
    {
        public const int PACKET_LENGTH = 61;
        public const int OSC_VALUES = 48;

        private volatile int[] data;
        int lastPos = 0;
        int lastUpdate = -1;
        int sequence = -1;

        long updates, time, dataAm;
        long starttime = DateTime.Now.Ticks;
        System.Windows.Forms.Timer t2;
        Random rand = new Random();
        public libusb_intf usb = new libusb_intf();

        Thread usbReadThread;

        public Scope() {
            libusb_intf.usb_set_debug(40);
            InitializeComponent();
            Application.ApplicationExit += new EventHandler(exit);

            data = new int[pictureBox1.Width];
            for (int i = 0; i < data.Length; i++) {
                data[i] = 0;
            }
#if WIN32
            DoubleBuffered = true;
#endif
        }

        private void exit(object sender, EventArgs e) {
            run = false;

            if (usb.device != IntPtr.Zero) {
                byte[] request = new byte[PACKET_LENGTH];
                request[0] = 16;
                byte[] reply = usb.usbSend(request, 63);
                reply = usb.usbReceive(63, false, true);
                libusb_intf.usb_close(usb.device);
            }
        }

        bool redraw = false;

        long lda;
        volatile int overflows, non_overflows;
        volatile bool run = true;

        private void timerTick2(object sender, EventArgs e) {
            t2.Stop();
            if (redraw) {
                if (!paused) {
                    pictureBox1.Invalidate();
                }
                time = (DateTime.Now.Ticks - starttime) / 10000;
                if (time != 0)
                    label1.Text = "Time:" + (time / 1000) + "s Fps:" + ((float)dataAm * 1000 / time) + " Overflows " + overflows + "/" + non_overflows + " Rtime:" + ((lastReceive - former) / 10000) + " Datapoints since last update:" + (dataAm - lda) * OSC_VALUES;

                lda = dataAm;

            }
            t2.Start();
        }

        long lastReceive, former;

        int selectedInRow, unselectedInRow;

        int skipped, noskipped;

        int avgAm, avgSum;

        private void timerTick() {

            while (run) {
                bool overflow = false;
                former = lastReceive;
                lastReceive = DateTime.Now.Ticks;


                byte[] reply = usb.usbReceive(63, true, false);
                if (reply == null)
                    return;

                sequence = reply[61];
                if (sequence == 255) {
                    sequence = -1;
                }

                //Console.WriteLine("osc received:" + usb.arrayToString(reply));
                //MessageBox.Show("received"+usb.arrayToString(reply));

                if ((reply[62] & 1) > 0) {
                    overflow = true;
                    overflows++;
                } else {
                    non_overflows++;
                }

                for (int i = 0; i < OSC_VALUES; i++) {
                    if (redraw && SPS >= 1000)
                        continue;
                    int value = reply[i];
                    int bytepos = OSC_VALUES + i / 4;
                    int bitpos = i % 4 * 2;
                    value = value + ((reply[bytepos] >> bitpos) & 3) * 256;
                    if ((skip > noSkip && (unselectedInRow < skip || selectedInRow >= noSkip)) ||
                         (skip <= noSkip && (unselectedInRow < skip && selectedInRow >= noSkip))) {

                        selectedInRow = 0;
                        unselectedInRow++;
                        skipped++;
                        avgAm++;
                        avgSum += value;
                        continue;
                    }

                    noskipped++;
                    unselectedInRow = 0;
                    selectedInRow++;

                    avgAm++;
                    avgSum += value;

                    lastPos++;
                    if (lastPos >= data.Length) {
                        lastPos = 0;
                        lastUpdate = -1;
                        redraw = true;
                    }
                    data[lastPos] = (int)Math.Round((double)(avgSum / avgAm));
                    avgAm = 0;
                    avgSum = 0;
                    if (SPS < 1000)
                        redraw = true;
                }
                if (overflow && data[lastPos] <= MAX_OSC) {
                    data[lastPos] += MAX_OSC + 1;
                    //lastPos = 0;
                }
                dataAm++;
            }
        }

        const int MAX_OSC = 1023;

        private int getY(int value, int height) {
            return (int)(height - (value - minv) * height / (maxv - minv) + s);
        }


        double[] changes = { 2, 2.5, 2 };

        const int s = 10; // screen starting position

        private double getSpacing(double pixelsPerUnit) {
            double spacing = 1;

            int changeState = 0;
            while (pixelsPerUnit > 0) {
                if (changeState < 0)
                    changeState = changes.Length - 1;
                if (changeState >= changes.Length)
                    changeState = 0;

                if (spacing * pixelsPerUnit > 125) {
                    spacing /= changes[changeState];
                    changeState--;
                } else if (spacing * pixelsPerUnit < 50) {
                    spacing *= changes[changeState];
                    changeState++;
                } else
                    break;
            }
            return spacing;
        }

        private double getStartValue(double min, double spacing) {
            int decimals = (int)Math.Log10(1 / spacing);
            return Math.Round(min, decimals);
        }

        private void drawGrid(Graphics g, int height, int width) {
            Pen gray = new Pen(Color.FromArgb(100, 100, 100));
            SolidBrush white = new SolidBrush(Color.White);
            if (max <= min || height < 2 || width < 2)
                return;


            double pixelsPerSecond = SPS;
            double xSpacing = getSpacing(pixelsPerSecond);
            for (double f = 0; f * pixelsPerSecond < width; f += xSpacing) {
                int x = (int)(f * pixelsPerSecond);
                if (x > width)
                    break;
                g.DrawLine(gray, x + s, 0, x + s, height + 2 * s);
                g.DrawString(f + "s", new Font("Arial", 13), white, x, height - 20);
            }

            double pixelsPerVolt = height / (max - min);
            double ySpacing = getSpacing(pixelsPerVolt);

            for (double f = getStartValue(min, ySpacing); f <= max; f += ySpacing) {
                int y = height - (int)((f - min) * pixelsPerVolt);
                g.DrawLine(gray, 0, y + s, width + 2 * s, y + s);
                double val = f;
                if (Math.Abs(f) < 0.00000000000001)
                    val = 0.0;
                g.DrawString(val + "V", new Font("Arial", 13), white, 0, y - 3);
            }

        }

        double max, maxv;
        double min, minv;

        private void pictureBox1_Paint(object sender, PaintEventArgs e) {
            lastUpdate = lastPos;
            int height = pictureBox1.Height - 4 - 2 * s;
            int width = pictureBox1.Width - 4 - 2 * s;

            max = (float)numericUpDownMax.Value;
            min = (float)numericUpDownMin.Value;
            minv = Math.Round(MAX_OSC * (float)zoomLow.Value / 100);
            maxv = Math.Round(MAX_OSC * (float)zoomHigh.Value / 100);

            if (autoZoom) {
                int minReal = MAX_OSC;
                int maxReal = 0;
                foreach (int item in data) {
                    int temp = item;
                    if (temp > MAX_OSC) {
                        temp -= (MAX_OSC + 1);
                    }
                    maxReal = (int)Math.Max(maxv, temp);
                    minReal = (int)Math.Min(minv, temp);
                }
                maxReal = Math.Min(maxReal, MAX_OSC);
                minReal = Math.Max(minReal, 0);
                minv = Math.Min(minv, minReal);
                maxv = Math.Max(maxv, maxReal);
                zoomLow.Value = (int)Math.Min(99, (minv * 100 / MAX_OSC));
                zoomHigh.Value = (int)Math.Max(1, (maxv * 100 / MAX_OSC));
            }


            double diff = (max - min);
            max = min + (maxv / MAX_OSC) * diff;
            min = min + (minv / MAX_OSC) * diff;


            drawGrid(e.Graphics, height, width);

            Point[] points = new Point[data.Length];
            int i = 0;
            Pen p = new Pen(Color.DarkBlue);
            foreach (int item in data) {
                int temp = item;
                if (item > MAX_OSC) {
                    temp -= (MAX_OSC + 1);
                    e.Graphics.DrawLine(p, i, 0, i, height);
                }
                points[i] = new Point(i, getY(temp, height));
                i++;
            }

            p = new Pen(Color.White);
            e.Graphics.DrawLines(p, points);
            p = new Pen(Color.DarkRed);
            e.Graphics.DrawLine(p, lastPos, 0, lastPos, height + s);
            p = new Pen(Color.Black);
            e.Graphics.DrawLine(p, lastPos + 1, 0, lastPos + 1, height);
            e.Graphics.DrawLine(p, lastPos - 1, 0, lastPos - 1, height);
            e.Graphics.DrawLine(p, lastPos - 2, 0, lastPos - 2, height);

            redraw = false;
            updates++;
        }

        private void button1_Click(object sender, EventArgs e) {
            updates = 0;
            starttime = DateTime.Now.Ticks;
            dataAm = 0;
        }

        private void pictureBox1_Resize(object sender, EventArgs e) {
            Array.Resize<int>(ref data, pictureBox1.Width);
            pictureBox1.Invalidate();
        }

        private void numericUpDownMax_ValueChanged(object sender, EventArgs e) {
            if (numericUpDownMax.Value <= numericUpDownMin.Value)
                numericUpDownMin.Value = numericUpDownMax.Value - 1;
            pictureBox1.Invalidate();
        }

        private void numericUpDownMin_ValueChanged(object sender, EventArgs e) {
            if (numericUpDownMax.Value <= numericUpDownMin.Value)
                numericUpDownMax.Value = numericUpDownMin.Value + 1;
            pictureBox1.Invalidate();

        }

        private void error(string err) {
            MessageBox.Show(err);
            Application.Exit();
        }


        private void Skooppi_Load(object sender, EventArgs e) {
            string str = usb.initUSB(5824, 1007);
            if (str != null) {
                error("Could not connect to oscilloscope...\n\"" + str + "\"");
            } else {
                byte[] request = new byte[PACKET_LENGTH];
                request[0] = 15;

                byte[] reply = usb.usbSend(request, 4);

                if (reply == null || reply[1] != 233) {
                    error("Error receiving...");
                    Application.Exit();
                    return;
                }
                /*
                t = new Timer();
                t.Interval = 1;
                t.Tick += new EventHandler(timerTick);
                t.Start();
                */

                usbReadThread = new Thread(timerTick);
                usbReadThread.Priority = ThreadPriority.Highest;
                usbReadThread.Start();

                t2 = new System.Windows.Forms.Timer();
                t2.Interval = 200;
                t2.Tick += new EventHandler(timerTick2);
                t2.Start();
            }
        }

        bool paused = false;

        private void button2_Click(object sender, EventArgs e) {
            paused = !paused;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e) {
        }

        volatile int SPS = 15479;
        volatile int skip = 0, noSkip = 1;

        private void trackBar1_Scroll(object sender, EventArgs e) {
            int oldSPS = SPS;
            SPS = (int)trackBar1.Value;
            double part = (double)SPS / trackBar1.Maximum;
            if (part < 0.45) {
                skip = (int)(1.0 / part);
                noSkip = 1;
            } else if (part == 1) {
                skip = 0;
                noSkip = 1;
            } else if (part < 0.55) {
                skip = 1;
                noSkip = 1;
            } else {
                skip = 1;
                noSkip = (int)(1.0 / (1 - part));
            }

            SPS = trackBar1.Maximum - (int)(trackBar1.Maximum * ((double)skip / (skip + noSkip)));
            if (SPS != oldSPS) {
                lastPos = 0;
                for (int i = 0; i < data.Length; i++) {
                    data[i] = 0;
                }
            }
            label5.Text = SPS.ToString();
            skipped = 0;
            noskipped = 0;

            pictureBox1.Invalidate();
        }

        private void numericUpDown1_ValueChanged_2(object sender, EventArgs e) {
            t2.Interval = (int)numericUpDown1.Value;
        }

        private void zoomHigh_Scroll(object sender, EventArgs e) {
            if (zoomHigh.Value <= zoomLow.Value)
                zoomLow.Value = zoomHigh.Value - 1;
            pictureBox1.Invalidate();
        }

        private void zoomLow_Scroll(object sender, EventArgs e) {
            if (zoomLow.Value >= zoomHigh.Value)
                zoomHigh.Value = zoomLow.Value + 1;
            pictureBox1.Invalidate();
        }

        bool autoZoom = false;

        private void checkBox1_CheckedChanged_1(object sender, EventArgs e) {
            autoZoom = checkBox1.Checked;
            pictureBox1.Invalidate();
        }

    }
}